@@ -1,3 +1,4 @@
+from datetime import datetime
 import logging
 import time
 import sys
@@ -5,17 +6,18 @@ import sys
 import boto3
 from boto3.s3.transfer import TransferConfig
 
-from zfs_uploader.backup_db import BackupDB
+from zfs_uploader.backup_db import BackupDB, DATETIME_FORMAT
 from zfs_uploader.snapshot_db import SnapshotDB
-from zfs_uploader.zfs import (create_filesystem, get_snapshot_send_size,
+from zfs_uploader.zfs import (destroy_filesystem, destroy_snapshot,
+                              get_snapshot_send_size,
                               get_snapshot_send_size_inc,
                               open_snapshot_stream,
-                              open_snapshot_stream_inc, ZFSError)
+                              open_snapshot_stream_inc, rollback_filesystem,
+                              ZFSError)
 
 KB = 1024
 MB = KB * KB
 S3_MAX_CONCURRENCY = 20
-S3_MAX_PART_NUMBER = 10000
 
 
 class BackupError(Exception):
@@ -88,6 +90,11 @@ class ZFSjob:
         """ S3 storage class. """
         return self._storage_class
 
+    @property
+    def max_multipart_parts(self):
+        """ Maximum number of parts to use in a multipart S3 upload. """
+        return self._max_multipart_parts
+
     @property
     def backup_db(self):
         """ BackupDB """
@@ -101,7 +108,7 @@ class ZFSjob:
     def __init__(self, bucket_name, access_key, secret_key, filesystem,
                  region=None, cron=None, max_snapshots=None, max_backups=None,
                  max_incremental_backups_per_full=None, storage_class=None,
-                 endpoint=None):
+                 endpoint=None, max_multipart_parts=None):
         """ Create ZFSjob object.
 
         Parameters
@@ -128,6 +135,8 @@ class ZFSjob:
             Maximum number of incremental backups per full backup.
         storage_class : str, default: STANDARD
             S3 storage class.
+        max_multipart_parts : int, default: 10000
+            Maximum number of parts to use in a multipart S3 upload.
 
         """
         self._bucket_name = bucket_name
@@ -150,6 +159,7 @@ class ZFSjob:
         self._max_backups = max_backups
         self._max_incremental_backups_per_full = max_incremental_backups_per_full # noqa
         self._storage_class = storage_class or 'STANDARD'
+        self._max_multipart_parts = max_multipart_parts or 10000
         self._logger = logging.getLogger(__name__)
 
         if max_snapshots and not max_snapshots >= 0:
@@ -216,7 +226,9 @@ class ZFSjob:
         Defaults to most recent backup if backup_time is not specified.
 
         WARNING: If restoring to a file system that already exists, snapshots
-        and data that were written after the backup will be destroyed.
+        and data that were written after the backup will be destroyed. The
+        file system will also be destroyed if there are no snapshots at any
+        point during the restore process.
 
         Parameters
         ----------
@@ -243,10 +255,57 @@ class ZFSjob:
         backup_type = backup.backup_type
         s3_key = backup.s3_key
 
-        if filesystem:
-            out = create_filesystem(filesystem)
-            if out.returncode:
-                raise ZFSError(out.stderr)
+        # Since we can't use the `-F` option with `zfs receive` for encrypted
+        # filesystems we have to handle removing filesystems, snapshots, and
+        # data written after the most recent snapshot ourselves.
+        if filesystem is None:
+            if snapshots:
+                # Destroy any snapshots that occurred after the backup
+                backup_datetime = datetime.strptime(backup_time,
+                                                    DATETIME_FORMAT)
+                for snapshot in snapshots:
+                    snapshot_datetime = datetime.strptime(snapshot,
+                                                          DATETIME_FORMAT)
+                    if snapshot_datetime > backup_datetime:
+                        self._logger.info(f'filesystem={self.filesystem} '
+                                          f'snapshot_name={backup_time} '
+                                          f's3_key={s3_key} '
+                                          f'msg="Destroying {snapshot} since '
+                                          'it occurred after the backup."')
+                        destroy_snapshot(backup.filesystem, snapshot)
+
+                self._snapshot_db.refresh()
+                snapshots = self._snapshot_db.get_snapshot_names()
+
+                # Rollback to most recent snapshot or destroy filesystem if
+                # there are no snapshots.
+                if snapshots:
+                    self._logger.info(f'filesystem={self.filesystem} '
+                                      f'snapshot_name={backup_time} '
+                                      f's3_key={s3_key} '
+                                      'msg="Rolling filesystem back to '
+                                      f'{snapshots[-1]}"')
+                    out = rollback_filesystem(backup.filesystem, snapshots[-1])
+                    if out.returncode:
+                        raise ZFSError(out.stderr)
+
+                    self._snapshot_db.refresh()
+                    snapshots = self._snapshot_db.get_snapshot_names()
+                else:
+                    self._logger.info(f'filesystem={self.filesystem} '
+                                      f'snapshot_name={backup_time} '
+                                      f's3_key={s3_key} '
+                                      'msg="Destroying filesystem since there '
+                                      'are no snapshots."')
+                    destroy_filesystem(backup.filesystem)
+
+            else:
+                self._logger.info(f'filesystem={self.filesystem} '
+                                  f'snapshot_name={backup_time} '
+                                  f's3_key={s3_key} '
+                                  'msg="Destroying filesystem since there are '
+                                  'no snapshots."')
+                destroy_filesystem(backup.filesystem)
 
         if backup_type == 'full':
             if backup_time in snapshots and filesystem is None:
@@ -284,7 +343,8 @@ class ZFSjob:
         filesystem = snapshot.filesystem
 
         send_size = int(get_snapshot_send_size(filesystem, backup_time))
-        transfer_config = _get_transfer_config(send_size)
+        transfer_config = _get_transfer_config(send_size,
+                                               self._max_multipart_parts)
 
         s3_key = f'{filesystem}/{backup_time}.full'
         self._logger.info(f'filesystem={filesystem} '
@@ -331,7 +391,8 @@ class ZFSjob:
         send_size = int(get_snapshot_send_size_inc(filesystem,
                                                    backup_time_full,
                                                    backup_time))
-        transfer_config = _get_transfer_config(send_size)
+        transfer_config = _get_transfer_config(send_size,
+                                               self._max_multipart_parts)
 
         s3_key = f'{filesystem}/{backup_time}.inc'
         self._logger.info(f'filesystem={filesystem} '
@@ -544,10 +605,10 @@ class TransferCallback:
             self._time_0 = time_1
 
 
-def _get_transfer_config(send_size):
+def _get_transfer_config(send_size, max_multipart_parts):
     """ Get transfer config. """
     # should never get close to the max part number
-    chunk_size = send_size // (S3_MAX_PART_NUMBER - 100)
+    chunk_size = send_size // (max_multipart_parts - 100)
     # only set chunk size if greater than default value
     chunk_size = chunk_size if chunk_size > 8 * MB else 8 * MB
     return TransferConfig(max_concurrency=S3_MAX_CONCURRENCY,
